"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const vscode_1 = require("vscode");
const typings_1 = require("./typings");
function compareMessages(m1, m2) {
    return (m1.file_name === m2.file_name &&
        m1.pos_line === m2.pos_line && m1.pos_col === m2.pos_col &&
        m1.severity === m2.severity && m1.caption === m2.caption && m1.text === m2.text);
}
// https://stackoverflow.com/questions/6234773/can-i-escape-html-special-chars-in-javascript
function escapeHtml(s) {
    return s
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#039;');
}
class InfoProvider {
    constructor(server, leanDocs, context, staticServer) {
        this.server = server;
        this.leanDocs = leanDocs;
        this.context = context;
        this.staticServer = staticServer;
        this.subscriptions = [];
        this.displayMode = typings_1.DisplayMode.AllMessage;
        this.statusShown = false;
        this.started = false;
        this.stopped = false;
        this.curFileName = null;
        this.curPosition = null;
        this.curGoalState = null;
        this.curMessages = null;
        this.curWidget = null;
        this.stylesheet = null;
        this.messageFormatters = [];
        this.statusBarItem = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Right, 1000);
        this.hoverDecorationType = vscode_1.window.createTextEditorDecorationType({
            backgroundColor: 'red',
            border: '3px solid red',
        });
        this.updateStylesheet();
        this.subscriptions.push(this.server.allMessages.on(() => {
            if (this.updateMessages()) {
                this.rerender();
            }
        }), this.server.statusChanged.on(() => __awaiter(this, void 0, void 0, function* () {
            if (this.displayMode === typings_1.DisplayMode.OnlyState) {
                const changed = yield this.updateGoal();
                if (changed) {
                    this.rerender();
                }
            }
        })), this.server.restarted.on(() => {
            this.autoOpen();
        }), vscode_1.window.onDidChangeActiveTextEditor(() => this.updatePosition(false)), vscode_1.window.onDidChangeTextEditorSelection(() => this.updatePosition(false)), vscode_1.workspace.onDidChangeConfiguration((e) => {
            this.updateStylesheet();
            this.rerender();
            if (!vscode_1.workspace.getConfiguration('lean').get('typeInStatusBar') && this.statusShown) {
                this.statusBarItem.hide();
                this.statusShown = false;
            }
        }), vscode_1.commands.registerCommand('_lean.revealPosition', this.revealEditorPosition.bind(this)), vscode_1.commands.registerCommand('_lean.infoView.pause', () => {
            this.stopUpdating();
        }), vscode_1.commands.registerCommand('_lean.infoView.continue', () => {
            this.setMode(this.displayMode);
        }), vscode_1.commands.registerTextEditorCommand('lean.displayGoal', (editor) => {
            this.setMode(typings_1.DisplayMode.OnlyState);
            this.openPreview(editor);
        }), vscode_1.commands.registerTextEditorCommand('lean.displayList', (editor) => {
            this.setMode(typings_1.DisplayMode.AllMessage);
            this.openPreview(editor);
        }), vscode_1.commands.registerTextEditorCommand('lean.infoView.displayGoal', (editor) => {
            this.setMode(typings_1.DisplayMode.OnlyState);
        }), vscode_1.commands.registerTextEditorCommand('lean.infoView.displayList', (editor) => {
            this.setMode(typings_1.DisplayMode.AllMessage);
        }), vscode_1.commands.registerTextEditorCommand('lean.infoView.copyToComment', (editor) => this.copyToComment(editor)), vscode_1.commands.registerTextEditorCommand('lean.infoView.toggleUpdating', (editor) => {
            if (this.stopped) {
                this.setMode(this.displayMode);
            }
            else {
                this.stopUpdating();
            }
        }));
        if (this.server.alive()) {
            this.autoOpen();
        }
    }
    dispose() {
        for (const s of this.subscriptions) {
            s.dispose();
        }
    }
    addMessageFormatter(f) {
        this.messageFormatters.push(f);
    }
    updateStylesheet() {
        const css = this.context.asAbsolutePath(path_1.join('media', 'infoview.css'));
        const fontFamily = 
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-type-assertion
        vscode_1.workspace.getConfiguration('editor').get('fontFamily').
            replace(/['"]/g, '');
        this.stylesheet = fs_1.readFileSync(css, 'utf-8') + `
            .font-code {
                font-family: ${fontFamily};
                font-size: ${vscode_1.workspace.getConfiguration('editor').get('fontSize')}px;
            }
            ` +
            vscode_1.workspace.getConfiguration('lean').get('infoViewStyle');
    }
    autoOpen() {
        if (!this.started && vscode_1.workspace.getConfiguration('lean').get('infoViewAutoOpen')) {
            this.started = true;
            this.setMode(vscode_1.workspace.getConfiguration('lean').get('infoViewAutoOpenShowGoal', true) ?
                typings_1.DisplayMode.OnlyState : typings_1.DisplayMode.AllMessage);
            this.openPreview(vscode_1.window.activeTextEditor);
            this.updatePosition(false);
        }
    }
    openPreview(editor) {
        let column = editor ? editor.viewColumn + 1 : vscode_1.ViewColumn.Two;
        if (column === 4) {
            column = vscode_1.ViewColumn.Three;
        }
        if (this.webviewPanel) {
            this.webviewPanel.reveal(column, true);
        }
        else {
            this.webviewPanel = vscode_1.window.createWebviewPanel('lean', this.displayMode === typings_1.DisplayMode.OnlyState ? 'Lean Goal' : 'Lean Messages', { viewColumn: column, preserveFocus: true }, {
                enableFindWidget: true,
                retainContextWhenHidden: true,
                enableScripts: true,
                enableCommandUris: true,
            });
            this.webviewPanel.webview.html = this.initialHtml();
            this.webviewPanel.onDidDispose(() => this.webviewPanel = null);
            this.webviewPanel.webview.onDidReceiveMessage((message) => this.handleMessage(message), undefined, this.subscriptions);
        }
    }
    /** Handle a message incoming from the webview. */
    handleMessage(message) {
        switch (message.command) {
            case 'selectFilter':
                vscode_1.workspace.getConfiguration('lean').update('infoViewFilterIndex', message.filterId, true);
                // the workspace configuration change already forces a rerender
                return;
            case 'hoverPosition':
                this.hoverEditorPosition(message.data[0], message.data[1], message.data[2], message.data[3], message.data[4]);
                return;
            case 'stopHover':
                this.stopHover();
                return;
            case 'widget_event':
                this.handleWidgetEvent(message);
                return;
        }
    }
    /** Runs whenever the user interacts with a widget. */
    handleWidgetEvent(message) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log("got widget event", message);
            message = Object.assign({ command: 'widget_event', file_name: this.curFileName, line: this.curPosition.line + 1, column: this.curPosition.character }, message);
            let result = yield this.server.send(message);
            console.log("recieved from server", result);
            if (!result.record) {
                return;
            }
            let record = result.record;
            if (record.status === "success" && record.widget) {
                this.curWidget = record.widget;
                this.rerender();
            }
            else if (record.status === "edit") {
                const new_command = record.action;
                this.curWidget = record.widget;
                for (const editor of vscode_1.window.visibleTextEditors) {
                    if (editor.document.fileName === message.file_name) {
                        const current_selection_range = editor.selection;
                        const cursor_pos = current_selection_range.active;
                        const prev_line = editor.document.lineAt(message.line - 2);
                        const spaces = prev_line.firstNonWhitespaceCharacterIndex;
                        const margin_str = [...Array(spaces).keys()].map(x => " ").join("");
                        // [hack] for now, we assume that there is only ever one command per line
                        // and that the command should be inserted on the line above this one.
                        yield editor.edit((builder) => {
                            builder.insert(prev_line.range.end, `\n${margin_str}${new_command}, `);
                        });
                        editor.selection = new vscode_1.Selection(message.line, spaces, message.line, spaces);
                    }
                }
            }
            else if (record.status === "invalid_handler") {
                console.warn(`No widget_event update for {${message.handler}, ${message.route}}: invalid handler.`);
                yield this.updateGoal();
                yield this.rerender();
            }
            else if (record.status === "error") {
                console.error(`Update gave an error: ${record.message}`);
            }
        });
    }
    /** post a position message to the webserver. */
    sendPosition() {
        this.postMessage({
            command: 'position',
            fileName: this.curFileName,
            line: this.curPosition.line + 1,
            column: this.curPosition.character,
        });
    }
    stopUpdating() {
        this.stopped = true;
        this.postMessage({ command: 'pause' });
    }
    rerender() {
        if (this.webviewPanel) {
            const infoViewTacticStateFilters = vscode_1.workspace.getConfiguration('lean').get('infoViewTacticStateFilters', []);
            const filterIndex = vscode_1.workspace.getConfiguration('lean').get('infoViewFilterIndex', -1);
            const cursorInfo = {
                widget: this.curWidget ? JSON.stringify(this.curWidget) : undefined,
                goalState: this.curGoalState,
                messages: this.curMessages,
                fileName: this.curFileName,
                displayMode: this.displayMode,
                filterIndex,
                infoViewTacticStateFilters,
                line: this.curPosition.line, column: this.curPosition.character,
                location_name: `${vscode_1.Uri.file(this.curFileName)}:${this.curPosition.line}:${this.curPosition.character}`,
                base_name: path_1.basename(this.curFileName),
            };
            this.postMessage({
                command: "sync",
                props: {
                    cursorInfo,
                    pinnedInfos: [],
                }
            });
        }
    }
    setMode(mode) {
        if (this.displayMode === mode && !this.stopped) {
            return;
        }
        this.displayMode = mode;
        if (this.webviewPanel) {
            this.webviewPanel.title = this.displayMode === typings_1.DisplayMode.OnlyState ? 'Lean Goal' : 'Lean Messages';
        }
        this.stopped = false;
        this.updatePosition(true);
    }
    postMessage(msg) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.webviewPanel) {
                return this.webviewPanel.webview.postMessage(msg);
            }
            else {
                return false;
            }
        });
    }
    revealEditorPosition(uri, line, column) {
        for (const editor of vscode_1.window.visibleTextEditors) {
            if (editor.document.uri.toString() === uri.toString()) {
                const pos = new vscode_1.Position(line - 1, column);
                editor.revealRange(new vscode_1.Range(pos, pos), vscode_1.TextEditorRevealType.InCenterIfOutsideViewport);
                editor.selection = new vscode_1.Selection(pos, pos);
            }
        }
    }
    hoverEditorPosition(uri, line, column, endLine, endColumn) {
        for (const editor of vscode_1.window.visibleTextEditors) {
            if (editor.document.uri.toString() === uri) {
                const pos = new vscode_1.Position(line - 1, column);
                const endPos = new vscode_1.Position(endLine - 1, endColumn);
                const range = new vscode_1.Range(pos, endPos);
                editor.setDecorations(this.hoverDecorationType, [range]);
            }
        }
    }
    stopHover() {
        for (const editor of vscode_1.window.visibleTextEditors) {
            if (editor.document.languageId === 'lean') {
                editor.setDecorations(this.hoverDecorationType, []);
            }
        }
    }
    changePosition() {
        if (!vscode_1.window.activeTextEditor ||
            !vscode_1.languages.match(this.leanDocs, vscode_1.window.activeTextEditor.document)) {
            return;
        }
        const oldFileName = this.curFileName;
        const oldPosition = this.curPosition;
        this.curFileName = vscode_1.window.activeTextEditor.document.fileName;
        this.curPosition = vscode_1.window.activeTextEditor.selection.active;
        return (this.curFileName !== oldFileName || !this.curPosition.isEqual(oldPosition));
    }
    updatePosition(forceRefresh) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stopped) {
                return;
            }
            const chPos = this.changePosition();
            if (!chPos && !forceRefresh) {
                return;
            }
            // clear type in status bar item
            this.statusBarItem.text = '';
            const chMsg = this.updateMessages();
            /* updateTypeStatus is only called from the cases of the following switch-block, so pausing
               live-updates to the infoview (via this.stopped) also pauses the type status bar item */
            switch (this.displayMode) {
                case typings_1.DisplayMode.OnlyState:
                    const chGoal = yield this.updateGoal();
                    if (chPos || chGoal || chMsg) {
                        this.rerender();
                    }
                    else if (forceRefresh) {
                        this.postMessage({ command: 'continue' });
                    }
                    break;
                case typings_1.DisplayMode.AllMessage:
                    if (vscode_1.workspace.getConfiguration('lean').get('typeInStatusBar')) {
                        const info = yield this.server.info(this.curFileName, this.curPosition.line + 1, this.curPosition.character);
                        this.updateTypeStatus(info);
                    }
                    if (forceRefresh || chMsg) {
                        this.rerender();
                    }
                    else {
                        this.sendPosition();
                    }
                    break;
            }
        });
    }
    updateMessages() {
        if (this.stopped || !this.curFileName) {
            return false;
        }
        let msgs;
        switch (this.displayMode) {
            case typings_1.DisplayMode.OnlyState:
                /* Heuristic: find first position to the left which has messages attached,
                   from that on show all messages in this line */
                msgs = this.server.messages
                    .filter((m) => m.file_name === this.curFileName &&
                    m.pos_line === this.curPosition.line + 1)
                    .sort((a, b) => a.pos_col - b.pos_col);
                if (!vscode_1.workspace.getConfiguration('lean').get('infoViewAllErrorsOnLine')) {
                    let startColumn;
                    let startPos = null;
                    for (let i = 0; i < msgs.length; i++) {
                        if (this.curPosition.character < msgs[i].pos_col) {
                            break;
                        }
                        if (this.curPosition.character === msgs[i].pos_col) {
                            startColumn = this.curPosition.character;
                            startPos = i;
                            break;
                        }
                        if (startColumn == null || startColumn < msgs[i].pos_col) {
                            startColumn = msgs[i].pos_col;
                            startPos = i;
                        }
                    }
                    if (startPos) {
                        msgs = msgs.slice(startPos);
                    }
                }
                break;
            case typings_1.DisplayMode.AllMessage:
                msgs = this.server.messages
                    .filter((m) => m.file_name === this.curFileName)
                    .sort((a, b) => a.pos_line === b.pos_line
                    ? a.pos_col - b.pos_col
                    : a.pos_line - b.pos_line);
                break;
        }
        if (!this.curMessages) {
            this.curMessages = msgs;
            return true;
        }
        const oldMsgs = this.curMessages;
        if (msgs.length === oldMsgs.length) {
            let eq = true;
            for (let i = 0; i < msgs.length; i++) {
                if (!compareMessages(msgs[i], oldMsgs[i])) {
                    eq = false;
                    break;
                }
            }
            if (eq) {
                return false;
            }
        }
        this.curMessages = msgs;
        return true;
    }
    updateGoal() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stopped || !this.curFileName || !this.curPosition) {
                return false;
            }
            let shouldUpdate = false;
            try {
                // get the 'save_info' format for this location.
                const info = yield this.server.info(this.curFileName, this.curPosition.line + 1, this.curPosition.character);
                const record = info.record;
                if (record && record.widget) {
                    this.curWidget = record.widget;
                    console.log("Found a widget");
                    console.log(record);
                    shouldUpdate = true;
                }
                else {
                    this.curWidget = null;
                }
                if (info.record && info.record.state) {
                    if (this.curGoalState !== info.record.state) {
                        this.curGoalState = info.record.state;
                        shouldUpdate = true;
                    }
                }
                else {
                    if (this.curGoalState) {
                        this.curGoalState = null;
                    }
                }
                if (vscode_1.workspace.getConfiguration('lean').get('typeInStatusBar')) {
                    this.updateTypeStatus(info);
                }
                return shouldUpdate;
            }
            catch (e) {
                if (e !== 'interrupted') {
                    throw e;
                }
            }
        });
    }
    updateTypeStatus(info) {
        if (!this.statusShown) {
            this.statusBarItem.show();
            this.statusShown = true;
        }
        if (info.record) {
            const name = info.record['full-id'] || info.record.text;
            if (name && !info.record.tactic_params) {
                this.statusBarItem.text = name + ' : ' + info.record.type;
            }
        }
    }
    getMediaPath(mediaFile) {
        // workaround for https://github.com/microsoft/vscode/issues/89038
        return this.staticServer.mkUri(path_1.join(this.context.extensionPath, 'media', mediaFile));
    }
    initialHtml() {
        return `
            <!DOCTYPE html>
            <html>
            <head>
                <meta charset="UTF-8" />
                <meta http-equiv="Content-type" content="text/html;charset=utf-8">
                <title>Infoview</title>
                <link rel="stylesheet" href="https://unpkg.com/tachyons/css/tachyons.min.css">
                <style>${this.stylesheet}</style>
            </head>
            <body>
                <div id="react_root"></div>
                <script src="${this.getMediaPath('index.js')}"></script>
            </body>
            </html>`;
    }
    copyToComment(editor) {
        return __awaiter(this, void 0, void 0, function* () {
            yield editor.edit((builder) => {
                builder.insert(editor.selection.end.with({ character: 0 }).translate({ lineDelta: 1 }), '/-\n' + this.renderText() + '\n-/\n');
            });
        });
    }
    renderText() {
        const msgText = this.curMessages &&
            this.curMessages.map((m) => `${path_1.basename(m.file_name)}:${m.pos_line}:${m.pos_col}: ${m.severity} ${m.caption}\n${m.text}`).join('\n');
        const goalText = this.curGoalState ? `Tactic State:\n${this.curGoalState}\n` : '';
        return goalText + msgText;
    }
}
exports.InfoProvider = InfoProvider;
//# sourceMappingURL=infoview.js.map