"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const vscode_1 = require("vscode");
const http_1 = require("http");
const path_1 = require("path");
// workaround for https://github.com/microsoft/vscode/issues/89038
// avoid vscode-resource:, let's just run our own http server...
class StaticServer {
    constructor(context) {
        this.context = context;
        this.app = express();
        this.key = Math.random().toString(36).substring(2);
        this.app.get(`/${this.key}/*`, (req, res) => {
            const fileName = req.params[0];
            if (this.isAllowedFilePath(fileName)) {
                res.sendFile(fileName);
            }
            else {
                res.sendStatus(403);
            }
        });
        this.server = http_1.createServer(this.app);
        this.server.listen(null, '127.0.0.1');
    }
    port() {
        return this.server.address().port;
    }
    isAllowedFilePath(fileName) {
        fileName = path_1.normalize(fileName);
        if (fileName.startsWith(this.context.extensionPath)) {
            return true;
        }
        for (const folder of vscode_1.workspace.workspaceFolders) {
            if (fileName.startsWith(folder.uri.fsPath)) {
                return true;
            }
        }
        return false;
    }
    mkUri(fileName) {
        return `http://127.0.0.1:${this.port()}/${this.key}/${encodeURI(fileName)}`;
    }
    dispose() {
        this.server.close();
    }
}
exports.StaticServer = StaticServer;
//# sourceMappingURL=staticserver.js.map